/*jslint plusplus: true */
/*global angular, cordova, navigator, Camera*/

'use strict';

angular.module('starter.controllers', [])
    .controller('ScriptsCtrl', function ($scope, $state, $sce, $rootScope, $ionicPlatform, $timeout, $ionicScrollDelegate, $ionicLoading,
        ScriptService, QuestService, ActionService) {
        $scope.showed = [];
        $scope.action = null;
        $scope.nextType = "text";
        $scope.nextStepText = '点击继续对话';
        $scope.nextStepVisable = true;
        $scope.history = {};
        var currentScript;

        function init() {
            var showHistory = function () {
                function updateUIFromHistory() {
                    $scope.history = ScriptService.getHistory();

                    if ($scope.history.showed !== null) {
                        $scope.showed = $scope.history.showed;
                        $ionicScrollDelegate.scrollBottom();
                    }
                    if ($scope.history.next !== null) {
                        ScriptService.setNextScriptId($scope.history.next);
                    }
                    if ($scope.history.action !== null) {
                        $scope.action = $scope.history.action;
                    }
                    if ($scope.history.nextStepText !== null) {
                        $scope.nextStepText = $scope.history.nextStepText;
                    }
                    if ($scope.history.nextStepVisable !== null) {
                        $scope.nextStepVisable = $scope.history.nextStepVisable;
                    }
                }
                if (Object.keys(ScriptService.getHistory()).length === 0) {
                    if (ScriptService.getNextScriptId() === 0) {
                        $scope.save();
                    } else {
                        ScriptService.initHistory(function () {
                            updateUIFromHistory();
                            $scope.showNext();
                        });
                    }
                } else {
                    updateUIFromHistory();
                }
            };

            if (QuestService.isQuestInited()) {
                var checkPoint = QuestService.getCheckPoint();
                $scope.actionBackground = checkPoint.background;
                $scope.npcAvatar = QuestService.npcAvatar();
                $scope.userAvatar = QuestService.userAvatar();
                showHistory();
            } else {
                $ionicLoading.show({template: "数据加载中 ..."});
                $ionicPlatform.ready(function () {
                    QuestService.initQuestInfo(function () {
                        var checkPoint = QuestService.getCheckPoint();
                        QuestService.setActionTitle(checkPoint.text);
                        $scope.actionBackground = checkPoint.background;
                        $scope.npcAvatar = QuestService.npcAvatar();
                        $scope.userAvatar = QuestService.userAvatar();
                        ScriptService.update(showHistory);
                        $ionicLoading.hide();
                    });
                });
            }
        }

        $scope.save = function () {
            $scope.history.showed = $scope.showed;
            $scope.history.next = ScriptService.getNextScriptId();
            $scope.history.action = $scope.action;
            $scope.history.nextStepText = $scope.nextStepText;
            $scope.history.nextStepVisable = $scope.nextStepVisable;
            ScriptService.updateHistory($scope.history);
        };

        $scope.showNext = function () {
            if (typeof ($scope.action) === 'function') { 
                $scope.action(currentScript);
                return;
            }

            var script = ScriptService.getNextScript();
            if ($scope.showed.indexOf(script) >= 0) {
                script = ScriptService.getNextScript();
            }
            currentScript = script;
            $scope.actionType = script.type;
            if (script !== null && script.type === 'text') {
                $scope.nextType = "text";
                $scope.showed.push(script);
                $scope.nextStepText = '点击继续对话';
            }
            if (script !== null && script.type !== 'text') {
                $scope.handleAction(script);
            }

            $scope.save();
            $timeout(function () {
                $ionicScrollDelegate.scrollBottom();
            }, 100);
        };

        $scope.handleAction = function (script) {
            $scope.nextType = "action";
            switch (script.type) {
            case "question": 
                $scope.action = $scope.showQuestion;
                $scope.nextStepText = script.text;
                $scope.buttonType = "text-button";
                $scope.actionStepText = "回答问题";
                break;
            case 'camera': 
                $scope.action = $scope.showCamera;
                $scope.nextStepText = script.text;
                $scope.actionStepText = $sce.trustAsHtml("<i class=\"icon ion-camera\"></i>");
                break;
            case 'checkin': 
                $scope.action = $scope.checkin;
                $scope.nextStepText = script.text || "到了到了~";
                $scope.actionStepText = "签到";
                $scope.buttonType = "text-button";
                break;
            case 'qrcode': 
                $scope.action = $scope.qrcode;
                $scope.nextStepText = script.text;
                $scope.actionStepText = $sce.trustAsHtml("<i class=\"icon ion-qr-scanner\"></i>");
                break;
            case 'extra': 
                $scope.action = $scope.extra;
                $scope.nextStepText = script.text;
                $scope.actionStepText = "我已完成";
                $scope.buttonType = "text-button";
                break;
            case 'finish': 
                $scope.action = $scope.finishCheckPoint;
                $scope.nextStepText = script.text;
                if (QuestService.getCheckPoint().isLastOfQuest) {
                    $scope.actionStepText = "结束任务";
                } else {
                    $scope.actionStepText = "马上去";
                }
                $scope.buttonType = "text-button";
                break;
            }
        };

        $scope.showQuestion = function (script) {
            $scope.action = null;
            ScriptService.shareScript(script);
            $state.go('question', {qId: script.questionId});

            $scope.nextStepText = '点击继续对话';
            $scope.save();
        };

        $scope.showCamera = function (script) {
            var submitPicture = function (image) {
                function onSuccess(attachment) {
                    function successAction(data) {
                        $scope.action = null;
                        $scope.nextStepText = '点击继续对话';
                        $scope.save();

                        setTimeout(function () {
                            $ionicLoading.hide();
                            $scope.showNext();
                        }, 0);
                    }
                    function failAction(data) {
                        $ionicLoading.hide();
                    }
                    var postData = {
                        action: "camera",
                        id: script.questionId,
                        attachment: {
                            attachment_id: attachment.id,
                            latitude: "0.00",
                            longitude: "0.00",
                            altitude: "0.00",
                            direction: "0.00"
                        }
                    };

                    $ionicLoading.show({template: "提交中..."});
                    ScriptService.submitAction(postData, successAction, failAction);
                }
                function onFail(message) {
                    console.log(message);
                    $ionicLoading.show({
                        template: "<i class=\"ion ion-alert-circled\"></i> 图片上传失败了，再试试~",
                        duration: 800
                    });
                }

                $ionicLoading.show({template: "照片压缩中 ..."});
                ActionService.uploadImage(image, onSuccess, onFail);
            };

            var takePicture = function () {
                function onSuccess(imageURI) {
                    submitPicture(imageURI);
                }

                function onFail(message) {
                    console.log(message);
                }

                navigator.camera.getPicture(onSuccess, onFail, { quality: 20,
                    destinationType: Camera.DestinationType.FILE_URI,
                     sourceType : Camera.PictureSourceType.CAMERA,
                      encodingType: Camera.EncodingType.JPEG,
                      targetWidth: 320,
                      targetHeight: 480
                    });
            };
            navigator.camera.cleanup();
            takePicture(); 
        };

        $scope.qrcode = function (script) {
            var scanSuccess, scanFailed;
            scanSuccess = function (result) {
                var postData = {
                    action: "qrcode",
                    id: script.questionId,
                    attachment: {
                        code: result
                    }
                };
                $ionicLoading.show({template: "提交中 ..."});
                ScriptService.submitAction(postData, function () {
                    $ionicLoading.hide();
                    $scope.action = null;
                    $scope.nextStepText = '点击继续对话';
                    $scope.save();
                    $scope.showNext();
                }, function (message) {
                    $ionicLoading.hide();
                    console.log(message);
                    $ionicLoading.show({
                        template: "<i class=\"ion ion-alert-circled\"></i> 失败了，再试试~",
                        duration: 800
                    });
                });
            };
            scanFailed = function (message) {
                $ionicLoading.show({
                    template: "<i class=\"ion ion-alert-circled\"></i> 失败了，再试试~",
                    duration: 800
                });
            };
            ActionService.scanQRCode(scanSuccess, scanFailed);
        };

        $scope.checkin = function (script) {
            $ionicLoading.show({template: "提交中 ..."});
            ActionService.currentLocation(function (location) {
                //console.log(location);
                $scope.action = null;
                location = location || {latitude: 0, longitude: 0};
                var postData = {
                    action: "checkin",
                    id: script.questionId,
                    attachment: {
                        latitude: location.latitude, 
                        longitude: location.longitude,
                        dateime: new Date()
                    }
                };
                ScriptService.submitAction(postData, function () {
                    $ionicLoading.hide();
                    $scope.nextStepText = '点击继续对话';
                    $scope.save();
                    $scope.showNext();
                }, function (message) {
                    $ionicLoading.hide();
                });
            }, function (message) {
                console.log(message);
                $ionicLoading.show({
                    template: "<i class=\"ion ion-alert-circled\"></i> 获取位置失败了，再试试~",
                    duration: 800
                });
            });
        };

        $scope.extra = function (script) {
            $scope.action = null;
            $ionicLoading.show({template: "提交中 ..."});
            var postData = {
                action: "extra",
                id: script.questionId,
                attachment: {
                }
            };
            ScriptService.submitAction(postData, function () {
                $ionicLoading.hide();
                $scope.nextStepText = '点击继续对话';
                $scope.save();
                $scope.showNext();
            }, function (message) {
                $ionicLoading.hide();
            });
        };

        $scope.finishCheckPoint = function (script) {
            $ionicLoading.show({template: "提交中 ..."});
            var postData = {
                action: "finish",
                id: script.questionId,
                attachment: {}
            };
            ScriptService.submitAction(postData, function () {
                ActionService.finishCheckPoint();
            }, function (message) {
                console.log(message);
            });
        };
        init();

        $rootScope.$on('showNext', function() {
            $scope.showNext();
        });
    })
    .controller('QuestionCtrl', function ($scope, $state, $stateParams, $timeout, $ionicLoading, 
        QuestService, ScriptService) {
        ScriptService.getQuestion($stateParams.qId, function (question) {
            $scope.npcAvatar = QuestService.npcAvatar();
            $scope.userAvatar = QuestService.userAvatar();
            var script = ScriptService.getShareScript();
            //console.log(question);
            $scope.question = question;
        }, function (message) {
        });
        $scope.history = ScriptService.getHistory();
        $scope.check_model = {selected: null};

        $scope.check = function ($id) {
            $scope.check_model = $scope.question.answers[$id];
            //console.log($scope.check_model);
        };

        $scope.submit = function () {
            //console.log($scope.check_model);
            if (!$scope.check_model.selected) {
                return;
            }

            function backToTalking() {
                var script = ScriptService.getNextScript();
                $scope.history.showed.push(script);
                $scope.history.next = ScriptService.getNextScriptId();
                ScriptService.updateHistory($scope.history);
                $scope.$emit('showNext');
                $state.go('action');
            }

            function onSuccess (data) {
                //console.log(data);
                backToTalking();
                $ionicLoading.hide();
            }

            function onFail (message) {
                console.log(message);
                $ionicLoading.hide();
            }

            var postData = {
                action: "question",
                id: $stateParams.qId,
                attachment: {
                    answer: $scope.check_model.selected.id
                }
            };

            $ionicLoading.show({template: "提交中 ..."});
            ScriptService.submitAction(postData, onSuccess, onFail);
        };
    });
