/*jslint plusplus: true */
/*global angular, cordova*/

'use strict';

var baseUrl = "http://banlv.xjimi.com/szww/",
    commonBaseUrl = "http://banlv.xjimi.com:8070/",
    auth = "",
    nextScriptId = 0,
    currentUser,
    currentCheckPoint,
    httpConfig = function () {
        return {
            headers: {
                Authorization: currentUser && currentUser.auth
            }
        };
    },
    finishStatus;

function getFullImageURL (imageURL, URLPrefix) {
    if (!imageURL) {
        return null;
    }
    imageURL = imageURL.trim();
    if (imageURL.indexOf("http://") == 0) {
        return imageURL;
    }
    URLPrefix = URLPrefix || baseUrl + "static/image/quest/";
    return URLPrefix + imageURL;
}

angular.module('starter.services', [])
    .factory('QuestService', ['$http', function ($http) {
        var questInited = false;

        return {
            initQuestInfo: function (onSuccess, onFail) {
                cordova.exec(
                    function (data) {
                        //console.log(data);
                        if (data) {
                            baseUrl = data.baseUrl;
                            commonBaseUrl = data.commonBaseUrl || commonBaseUrl;

                            currentUser = data.user;
                            currentUser.avatar = getFullImageURL(currentUser.profile.avatar, commonBaseUrl + "static/image/avatar/");
                            currentUser.auth = "banlv " + currentUser.token;

                            currentCheckPoint = data.checkPoint;
                            currentCheckPoint.background = getFullImageURL(currentCheckPoint.background);
                            currentCheckPoint.avatar = getFullImageURL(currentCheckPoint.avatar);
                            nextScriptId = currentCheckPoint.nextScriptId;

                            questInited = true;

                            if (typeof onSuccess === "function") {
                                onSuccess();
                            }
                        } else {
                            if (typeof onFail === "function") {
                                onFail("no data returned.");
                            }
                        }
                    },
                    function (message) {
                        console.log("getInitQuestInfo failed: ");
                        console.log(message);
                    },
                    "QuestHelper",
                    "getInitQuestInfo",
                    []
                );
            },
            isQuestInited: function () {
                return questInited;
            },
            getUser: function () {
                return currentUser;
            },
            userAvatar: function () {
                return currentUser.avatar;
            },
            npcAvatar: function () {
                return currentCheckPoint.avatar;
            },
            getCheckPoint: function () {
                return currentCheckPoint;
            },
            remove: function () {
                var url = baseUrl + "assignment/remove";
                $http.get(url, httpConfig());
            },
            setActionTitle: function (title) {
                cordova.exec(
                    function () {

                    }, function () {

                    },
                    "QuestHelper",
                    "setActionTitle",
                    [title]
                );
            }
        };
    }])
    .factory('ScriptService', ['$http', function ($http) {
        var scripts = [],
            history = {},
            sharedScript,
            me = {
                all: function () {
                    return scripts;
                },
                update: function (callback) {
                    var url = baseUrl + "assignment/get_script";
                    $http.get(url, httpConfig())
                        .success(function (data) {
                            //console.log(data);
                            scripts = [];
                            data.scripts.forEach(function (script) {
                                var newScript;
                                if (script.user) {
                                    newScript = {
                                        role: "user",
                                        type: "text",
                                        text: script.user,
                                        avatar: currentUser.avatar
                                    };
                                } else if (script.action) {
                                    var results = script.action.match(/(.*)\|(.*)\|(.*)/i);
                                    if (results) {
                                        newScript = {
                                            text: results[1],
                                            type: results[2],
                                            questionId: results[3]
                                        };
                                    }
                                } else {
                                    newScript = {
                                        role: "npc",
                                        type: "text",
                                        text: script.npc,
                                        avatar: getFullImageURL(script.avatar) || currentCheckPoint.avatar
                                    };
                                }
                                if (newScript) {
                                    scripts.push(newScript);
                                }
                            });
                            // console.log(scripts);
                            callback();
                        })
                        .error(function (message) {
                            console.log(message);
                            callback(message);
                        });
                },
                shareScript: function (script) {
                    sharedScript = script;
                },
                getShareScript: function () {
                    return sharedScript;
                },
                get: function (sId) {
                    return scripts[sId];
                },
                setNextScriptId: function (sId) {
                    nextScriptId = sId;
                },
                getNextScriptId: function () {
                    return nextScriptId;
                },
                getNextScript: function () {
                    if (nextScriptId < scripts.length) {
                        return scripts[nextScriptId++];
                    }
                },
                getActionAnswer: function (actionType) {
                    if (nextScriptId <= scripts.length) {
                        return me.getNextScript();
                    }
                    return {};
                },
                getQuestion: function (qId, onSuccess, onFail) {
                    var questionUrl = baseUrl + "rest/action/" + qId;
                    $http.get(questionUrl, httpConfig())
                        .success(function (data) {
                            var question = data;
                            question.attachment = baseUrl + "static/image/quest/" + question.attachment;
                            var url = baseUrl + "rest/action_response";
                            var config = httpConfig();
                            config.params = {
                                "q": {
                                    "filters": [
                                        { "name": "action_id", "op": "eq", "val": qId }
                                    ]
                                }
                            };
                            $http.get(url, config)
                                .success(function (data) {
                                    //console.log(data.objects);
                                    question.answers = data.objects;
                                    if (typeof onSuccess === 'function') {
                                        onSuccess(question);
                                    }
                                }).error(function (message) {
                                    console.log("error at getQuestion: ");
                                    console.log(message);
                                    if (typeof onFail === 'function') {
                                        onFail(message);
                                    }
                                });
                        })
                        .error(function (message) {
                            console.log("error at getQuestion: ");
                            console.log(message);
                            if (typeof onFail === 'function') {
                                onFail(message);
                            }
                        });
                },
                getHistory: function () {
                    return history;
                },
                initHistory: function (callback) {
                    var script, count = 0;
                    if (nextScriptId > 0) {
                        nextScriptId --;
                        history.showed = [];
                        while (count < nextScriptId && count < scripts.length) {
                            script = scripts[count];
                            if (script && script.type === "text") {
                                history.showed.push(script);
                            }
                            count++;
                        }
                    }
                    history.next = nextScriptId;
                    callback(history);
                },
                updateHistory: function (c) {
                    history = c;
                    cordova.exec(
                        function () {
                        },
                        function (message) {
                            console.log(message);
                        },
                        "QuestHelper",
                        "saveCheckPoint",
                        [nextScriptId]
                    );
                },
                submitAction: function (postData, onSuccess, onFail) {
                    var url = baseUrl + "assignment/action";
                    //console.log(postData);
                    $http.post(url, postData, httpConfig())
                        .success(function (data) {
                            if (data.ret !== 0) {
                                onFail(data);
                                return;
                            }
                            console.log(data);
                            if (data && data.status) {
                                finishStatus = data.status;
                            } else {
                                finishStatus = {}
                            }
                            cordova.exec(
                                function () {
                                },
                                function (message) {
                                },
                                "QuestHelper",
                                "updateScore",
                                [data.total_credit]
                            );
                            me.update(onSuccess);
                        })
                        .error(function (message) {
                            console.log(message);
                            onFail(message);
                        });
                }
            };
        return me;
    }])
    .factory('ActionService', [function () {
        return {
            uploadImage: function (imagePath, onSuccess, onFail) {
                cordova.exec(
                    function (name) {
                        //console.log(name);
                        onSuccess(name);
                    },
                    function (message) {
                        console.log(message);
                        onFail(message);
                    },
                    "QuestHelper",
                    "uploadImage",
                    [imagePath]
                );
            },
            currentLocation: function (onSuccess, onFail) {
                cordova.exec(
                    function (location) {
                        //console.log(location);
                        onSuccess(location);
                    },
                    function (message) {
                        onFail(message);
                        console.log(message);
                    },
                    "QuestHelper",
                    "getLocation",
                    []
                );
            },
            finishCheckPoint: function () {
                cordova.exec(
                    function () {
                    },
                    function (message) {
                        console.log(message);
                    },
                    "QuestHelper",
                    "finishCheckPoint",
                    [finishStatus]
                );
            },
            scanQRCode: function (onSuccess, onFail) {
                // scanQRCode
                cordova.exec(
                    function (result) {
                        console.log(result);
                        onSuccess(result);
                    },
                    function (message) {
                        console.log(message);
                        onFail(message);
                    },
                    "QuestHelper",
                    "scanQRCode",
                    []
                );
            }
        };
    }]);
